/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.wmsclient;

/**
 * Estructura de datos que almacena la informacin necesaria para realizar una
 * peticin GetFeatureInfo
 *
 * @author Fernando Gonzlez Corts
 */
public class FeatureInfoQuery {
    private MapQuery mq;
    private String infoFormat;
    private int featureCount = -1;
    private String layerInfoQuery;
    private int x;
    private int y;
    private String exceptions;

    /**
     * Creates a new FeatureInfoQuery object.
     *
     * @param query MapQuery que se embeber en la peticin GetFeatureInfo
     */
    public FeatureInfoQuery(MapQuery query) {
        mq = query;
    }

    /**
     * Obtiene la representacin de la query en la forma de las peticiones GET
     *
     * @return string
     */
    public String getQuery() {
        String ret = "";
        ret = ret + mq.getQueryHeader() +
            "REQUEST=GetFeatureInfo&" + mq.getPartialQuery()+ 
			"&QUERY_LAYERS=" + layerInfoQuery + 
			"&X=" + x + "&Y=" + y;

        if (infoFormat != null) {
            ret += ("&INFO_FORMAT=" + infoFormat);
        }

        if (featureCount != -1) {
            ret += ("&FEATURE_COUNT=" + featureCount);
        }

        if (exceptions != null) {
            ret += ("&EXCEPTIONS=" + exceptions);
        } else {
            ret += ("&EXCEPTIONS=XML");
        }

        return ret;
    }

    /**
     * Obtiene el parmetro exceptions
     *
     * @return
     */
    public String getExceptions() {
        return exceptions;
    }

    /**
     * Obtiene el parmetro FeatureCount
     *
     * @return
     */
    public int getFeatureCount() {
        return featureCount;
    }

    /**
     * Obtiene el parmetro InfoFormat
     *
     * @return
     */
    public String getInfoFormat() {
        return infoFormat;
    }

    /**
     * Obtiene la MapQuery embebida
     *
     * @return
     */
    public MapQuery getMq() {
        return mq;
    }

    /**
     * Obtiene el parmetro X
     *
     * @return
     */
    public int getX() {
        return x;
    }

    /**
     * Obtiene el parmetro Y
     *
     * @return
     */
    public int getY() {
        return y;
    }

    /**
     * Establece el parmetro Exceptions
     *
     * @param string
     */
    public void setExceptions(String string) {
        exceptions = string;
    }

    /**
     * Establece el parmetro FeatureCount
     *
     * @param i
     */
    public void setFeatureCount(int i) {
        featureCount = i;
    }

    /**
     * Establece el parmetro InfoFormat
     *
     * @param string
     */
    public void setInfoFormat(String string) {
        infoFormat = string;
    }

    /**
     * Establece la MapQuery embebida
     *
     * @param query
     */
    public void setMq(MapQuery query) {
        mq = query;
    }

    /**
     * Establece el parmetro X
     *
     * @param i
     */
    public void setX(int i) {
        x = i;
    }

    /**
     * Establece el parmetro Y
     *
     * @param i
     */
    public void setY(int i) {
        y = i;
    }

    /**
     * Devuelve una string con los nombres de las capas queryables separadas
     * por comas
     *
     * @return
     */
    public String getInfoQuery() {
        return layerInfoQuery;
    }

    /**
     * Establece los nombres de las capas queryables
     *
     * @param string string con los nombres separados por comas
     */
    public void setInfoQuery(String string) {
        layerInfoQuery = string;
    }
}
