/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.wmsclient;

import java.awt.geom.Rectangle2D;


/**
 * Estructura de datos para almacenar los parmetros de una peticin GetMap
 *
 * @author Fernando Gonzlez Corts
 */
public class MapQuery {
	private String queryHeader;
    private String layers;
    private String styles;
    private String SRS;
    private String BBOX;
    private int width;
    private int height;
    private String format;
    private boolean transparent = true;
    private String BGColor = null;
    private String exceptions = null;
    private String time = null;
    private int elevation = -1;

    /**
     * Creates a new MapQuery object.
     */
    public MapQuery(String queryHeader) {
    	this.queryHeader = queryHeader;
    }

    /**
     * Obtiene la representacin de la query en la forma de las peticiones GET
     *
     * @return string
     */
    public String getQuery() {
        String ret = "";
        ret = ret + queryHeader + "REQUEST=GetMap&SERVICE=WMS&";
        ret = ret + getPartialQuery();
        if (exceptions != null) {
            ret += ("&EXCEPTIONS=" + exceptions);
        } else {
            ret += ("&EXCEPTIONS=XML");
        }


        return ret;
    }

    /**
     * Obtiene la query que se comparte con la peticin de 
     * getFeatureInfo
     *
     * @return String con la query
     */
    public String getPartialQuery() {
        String ret = "LAYERS=" + layers + "&STYLES=" + styles + "&SRS=" + SRS +
            "&BBOX=" + BBOX + "&WIDTH=" + width + "&HEIGHT=" + height +
            "&FORMAT=" + format;

        if (transparent) {
            ret += "&TRANSPARENT=TRUE";
        }

        if (BGColor != null) {
            ret += ("&COLOR=" + BGColor);
        }

        if (time != null) {
            ret += ("&TIME=" + time);
        }

        if (elevation != -1) {
            ret += ("&ELEVATION=" + elevation);
        }

        return ret;
    }

    /**
     * Obtiene el parmetro BBOX
     *
     * @return
     */
    public String getBBOX() {
        return BBOX;
    }

    /**
     * Obtiene el parmetro
     *
     * @return
     */
    public String getBGColor() {
        return BGColor;
    }

    /**
     * Obtiene el parmetro
     *
     * @return
     */
    public int getElevation() {
        return elevation;
    }

    /**
     * Obtiene el parmetro
     *
     * @return
     */
    public String getExceptions() {
        return exceptions;
    }

    /**
     * Obtiene el parmetro
     *
     * @return
     */
    public String getFormat() {
        return format;
    }

    /**
     * Obtiene el parmetro
     *
     * @return
     */
    public int getHeight() {
        return height;
    }

    /**
     * Obtiene el parmetro
     *
     * @return
     */
    public String getLayers() {
        return layers;
    }

    /**
     * Obtiene el parmetro
     *
     * @return
     */
    public String getSRS() {
        return SRS;
    }

    /**
     * Obtiene el parmetro
     *
     * @return
     */
    public String getStyles() {
        return styles;
    }

    /**
     * Obtiene el parmetro
     *
     * @return
     */
    public String getTime() {
        return time;
    }

    /**
     * Obtiene el parmetro
     *
     * @return
     */
    public boolean isTransparent() {
        return transparent;
    }

    /**
     * Obtiene el parmetro
     *
     * @return
     */
    public int getWidth() {
        return width;
    }

    /**
     * Establece el parmetro bbox
     *
     * @param minx
     * @param miny
     * @param maxx
     * @param maxy
     */
    public void setBBOX(String minx, String miny, String maxx, String maxy) {
        BBOX = minx + "," + miny + "," + maxy + "," + maxy;
    }

    /**
     * Establece el parmetro bbox
     *
     * @param rect
     */
    public void setBBOX(Rectangle2D rect) {
        BBOX = rect.getMinX() + "," + rect.getMinY() + "," + rect.getMaxX() +
            "," + rect.getMaxY();
    }

    /**
     * Establece el parmetro
     *
     * @param color
     */
    public void setBGColor(String color) {
        BGColor = color;
    }

    /**
     * Establece el parmetro
     *
     * @param i
     */
    public void setElevation(int i) {
        elevation = i;
    }

    /**
     * Establece el parmetro
     *
     * @param string
     */
    public void setExceptions(String string) {
        exceptions = string;
    }

    /**
     * Establece el parmetro
     *
     * @param string
     */
    public void setFormat(String string) {
        format = string;
    }

    /**
     * Establece el parmetro
     *
     * @param i
     */
    public void setHeight(int i) {
        height = i;
    }

    /**
     * Establece el parmetro
     *
     * @param string
     */
    public void setLayers(String string) {
        layers = string;
    }

    /**
     * Establece el parmetro
     *
     * @param string
     */
    public void setSRS(String string) {
        SRS = string;
    }

    /**
     * Establece el parmetro
     *
     * @param string
     */
    public void setStyles(String string) {
        styles = string;
    }

    /**
     * Establece el parmetro
     *
     * @param date
     */
    public void setTime(String date) {
        time = date;
    }

    /**
     * Establece el parmetro
     *
     * @param b
     */
    public void setTransparent(boolean b) {
        transparent = b;
    }

    /**
     * Establece el parmetro
     *
     * @param i
     */
    public void setWidth(int i) {
        width = i;
    }

    /**
     * Establece el parmetro
     *
     * @param string
     */
    public void setBBOX(String string) {
        BBOX = string;
    }
	/**
	 * @return Returns the queryHeader.
	 */
	public String getQueryHeader() {
		return queryHeader;
	}
	/**
	 * @param queryHeader The queryHeader to set.
	 */
	public void setQueryHeader(String queryHeader) {
		this.queryHeader = queryHeader;
	}
}
