/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.wmsclient;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import java.rmi.NoSuchObjectException;


/**
 * Clase con mtodos de utilidad en el protocolo WMS
 *
 * @author Fernando Gonzlez Corts
 */
public class WMSUtilities {
    /**
     * Copia el contenido de un InputStream en un OutputStream
     *
     * @param in InputStream
     * @param out OutputStream
     */
    public static void serializar(InputStream in, OutputStream out) {
        byte[] buffer = new byte[10240];

        int n;

        try {
            while ((n = in.read(buffer)) != -1) {
                out.write(buffer, 0, n);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Elimina del xml la declaracin del DTD
     *
     * @param bytes bytes del fichero XML de respuesta a getCapabilities
     * @param startTag Tag raiz del xml respuesta a getCapabilities
     *
     * @return bytes del fichero XML sin la declaracin del DTD
     */
    public static byte[] eliminarDTD(byte[] bytes, String startTag) {
        String text = new String(bytes);
        int index1 = text.indexOf("?>") + 2;
        int index2;

        try {
            index2 = findBeginIndex(bytes, startTag);
        } catch (NoSuchObjectException e) {
            return bytes;
        }

        byte[] buffer = new byte[bytes.length - (index2 - index1)];
        System.arraycopy(bytes, 0, buffer, 0, index1);
        System.arraycopy(bytes, index2, buffer, index1, bytes.length - index2);

        return buffer;
    }

    /**
     * Obtiene el ndice del comienzo del xml
     *
     * @param bytes bytes del fichero XML en el que se busca
     * @param tagRaiz Tag raiz del xml respuesta a getCapabilities
     *
     * @return ndice donde empieza el tag raiz
     *
     * @throws NoSuchObjectException Si no se encuentra el tag
     */
    private static int findBeginIndex(byte[] bytes, String tagRaiz)
        throws NoSuchObjectException {
        try {
            int nodo = 0;
            int ret = -1;

            int i = 0;

            while (true) {
                switch (nodo) {
                    case 0:

                        if (bytes[i] == '<') {
                            ret = i;
                            nodo = 1;
                        }

                        break;

                    case 1:

                        if (bytes[i] == ' ') {
                        } else if (bytes[i] == tagRaiz.charAt(0)) {
                            nodo = 2;
                        } else {
                            nodo = 0;
                        }

                        break;

                    case 2:

                        String aux = new String(bytes, i, 18);

                        if (aux.equalsIgnoreCase(tagRaiz.substring(1))) {
                            return ret;
                        }

                        nodo = 0;

                        break;
                }

                i++;
            }
        } catch (Exception e) {
            throw new NoSuchObjectException("No se pudo parsear el xml");
        }
    }
}
