/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.wmsclient.wms_1_0_0;

import com.iver.utiles.StringUtilities;

import com.iver.wmsclient.Capabilities;
import com.iver.wmsclient.FeatureInfoQuery;
import com.iver.wmsclient.MapQuery;
import com.iver.wmsclient.UnsupportedVersionException;
import com.iver.wmsclient.WMSException;
import com.iver.wmsclient.WMSUtilities;
import com.iver.wmsclient.WMSVersionClient;
import com.iver.wmsclient.wms_1_0_0.capabilities.BoundingBox;
import com.iver.wmsclient.wms_1_0_0.capabilities.DCPType;
import com.iver.wmsclient.wms_1_0_0.capabilities.FeatureInfo;
import com.iver.wmsclient.wms_1_0_0.capabilities.Format;
import com.iver.wmsclient.wms_1_0_0.capabilities.LatLonBoundingBox;
import com.iver.wmsclient.wms_1_0_0.capabilities.Layer;
import com.iver.wmsclient.wms_1_0_0.capabilities.WMT_MS_Capabilities;
import com.iver.wmsclient.wms_1_0_0.exception.ServiceException;
import com.iver.wmsclient.wms_1_0_0.exception.ServiceExceptionReport;

import org.apache.log4j.Logger;

import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Unmarshaller;
import org.exolab.castor.xml.ValidationException;

import java.awt.geom.Rectangle2D;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import java.net.URL;

import java.util.ArrayList;


/**
 * Cliente para la versin 1.0.0
 *
 * @author Fernando Gonzlez Corts
 */
public class WMS_1_0_0 implements WMSVersionClient {
    private static Logger logger = Logger.getLogger(WMS_1_0_0.class.getName());
    private String[] mapURLs;
    private String[] infoURLs;
    private WMT_MS_Capabilities root;

    /**
     * @see com.iver.cit.gvsig.fmap.wms.wmsclient.WMSClient#getCapabilities(java.net.URL)
     */
    public Capabilities getCapabilities(URL host)
        throws UnsupportedVersionException, IllegalStateException, IOException {
        InputStream in = null;
        String response = null;

        try {
            logger.debug("Obteniendo capabilities... host=" + host.toString());

            String hostString = host.toString();

            if (hostString.indexOf("?") == -1)
            {
                // Viene sin el interrogante
                hostString += "?";
            }
            else
            {
                if (!hostString.endsWith("?")) {
                    hostString += "&";
                }
            }

            String request = getQueryHeader() + "request=capabilities";

            URL query = new URL(hostString + request);
            in = query.openStream();

            ByteArrayOutputStream outbytes = new ByteArrayOutputStream();
            WMSUtilities.serializar(in, outbytes);

            response = new String(outbytes.toByteArray());
            logger.debug(response);

            logger.debug("Parseando la respuesta...");

            try {
                Unmarshaller umn = new Unmarshaller(WMT_MS_Capabilities.class);
                umn.setIgnoreExtraElements(true);
                umn.setIgnoreExtraAttributes(true);
                root = (WMT_MS_Capabilities) umn.unmarshal(new InputStreamReader(
                            new ByteArrayInputStream(WMSUtilities.eliminarDTD(
                                    outbytes.toByteArray(),
                                    "WMT_MS_Capabilities"))));
            } catch (MarshalException e) {
                Capabilities c = new Capabilities();
                c.setVersion(null);

                return c;
            }

            //			LocalConfiguration.getInstance().getProperties().setProperty("org.exolab.castor.xml.strictelements", "false");

            /*                        WMT_MS_Capabilities root = (WMT_MS_Capabilities) WMT_MS_Capabilities.unmarshal(new InputStreamReader(
               new ByteArrayInputStream(WMSUtilities.eliminarDTD(
                               outbytes.toByteArray()))));
             */
            DCPType[] DCPs = root.getCapability().getRequest().getMap()
                                 .getDCPType();
            mapURLs = new String[DCPs.length];

            for (int i = 0; i < DCPs.length; i++) {
                mapURLs[i] = DCPs[i].getHTTP().getGet().getOnlineResource();

                if (!mapURLs[i].endsWith("?")) {
                    mapURLs[i] = mapURLs[i] + "?";
                }
            }

            FeatureInfo fi = root.getCapability().getRequest().getFeatureInfo();

            if (fi != null) {
                DCPs = fi.getDCPType();
                infoURLs = new String[DCPs.length];

                for (int i = 0; i < DCPs.length; i++) {
                    infoURLs[i] = DCPs[i].getHTTP().getGet().getOnlineResource();

                    if (!infoURLs[i].endsWith("?")) {
                        infoURLs[i] = infoURLs[i] + "?";
                    }
                }
            }

            Capabilities ret = new Capabilities();
            ret.setRoot(root);
            ret.setVersion(root.getVersion());

            return ret;
        } catch (IOException e) {
            logger.debug(e);

            throw e;
        } catch (Exception e) {
            logger.error("Error", e);

            Capabilities ret = new Capabilities();

            if (response == null) {
                ret.setRoot(null);
                ret.setVersion(null);

                return ret;
            } else {
                ret.setRoot(null);
                ret.setVersion(StringUtilities.substringDelimited(response,
                        "version=\"", "\"", response.indexOf("bilities")));

                return ret;
            }
        } finally {
            if (in != null) {
                in.close();
            }
        }
    }

    /**
     * @see com.iver.cit.gvsig.fmap.wms.wmsclient.WMSClient#doMapQuery(com.iver.cit.gvsig.fmap.wms.wmsclient.MapQuery)
     */
    public byte[] doMapQuery(MapQuery queryInfo)
        throws IOException, NoSuchFieldException, ValidationException, 
            WMSException {
        throw new RuntimeException("No debi llegar nunca a esta linea");
    }

    /**
     * @see com.iver.cit.gvsig.fmap.wms.wmsclient.WMSClient#getVersion()
     */
    public String getVersion() {
        return "1.0.0";
    }

    /**
     * @see com.iver.cit.gvsig.fmap.wms.wmsclient.WMSClient#getBoundingBox(javax.swing.tree.TreePath,
     *      java.lang.String)
     */
    public Rectangle2D getBoundingBox(String[] t, String srs) {
        Layer[] layer = getLayerByPath(t);
        BoundingBox[] bbox = getBoundingBox(layer);

        for (int i = bbox.length - 1; i >= 0; i--) {
            if (bbox[i].getSRS().equals(srs)) {
                return new Rectangle2D.Double(Double.parseDouble(
                        bbox[i].getMinx()),
                    Double.parseDouble(bbox[i].getMiny()),
                    Double.parseDouble(bbox[i].getMaxx()) - Double.parseDouble(bbox[i].getMinx()),
                    Double.parseDouble(bbox[i].getMaxy()) - Double.parseDouble(bbox[i].getMiny()));
            }
        }

        LatLonBoundingBox[] llbbox = getLatLonBoundingBox(layer);

        for (int i = llbbox.length - 1; i >= 0; i--) {
            return new Rectangle2D.Double(Double.parseDouble(
                    llbbox[i].getMinx()),
                Double.parseDouble(llbbox[i].getMiny()),
                Double.parseDouble(llbbox[i].getMaxx()) - Double.parseDouble(
                        llbbox[i].getMinx()),
                Double.parseDouble(llbbox[i].getMaxy()) - Double.parseDouble(llbbox[i].getMiny()));
        }

        return null;
    }

    /**
     * Dado un array de layers devuelve el array de boundingbox de  todos ellos
     *
     * @param layer Array de layers
     *
     * @return array de boundingBox
     */
    private BoundingBox[] getBoundingBox(Layer[] layer) {
        ArrayList globalBbox = new ArrayList();

        for (int i = 0; i < layer.length; i++) {
            Layer l = (Layer) layer[i];
            BoundingBox[] bboxes = l.getBoundingBox();

            if (bboxes != null) {
                for (int j = 0; j < bboxes.length; j++) {
                    globalBbox.add(bboxes[j]);
                }
            }
        }

        return (BoundingBox[]) globalBbox.toArray(new BoundingBox[0]);
    }

    /**
     * Dado un array de layers devuelve el array de boundingbox de  todos ellos
     *
     * @param layer Array de layers
     *
     * @return array de boundingBox
     */
    private LatLonBoundingBox[] getLatLonBoundingBox(Layer[] layer) {
        ArrayList globalBbox = new ArrayList();

        for (int i = 0; i < layer.length; i++) {
            Layer l = (Layer) layer[i];
            LatLonBoundingBox bbox = l.getLatLonBoundingBox();

            if (bbox != null) {
                globalBbox.add(bbox);
            }
        }

        return (LatLonBoundingBox[]) globalBbox.toArray(new LatLonBoundingBox[0]);
    }

    /**
     * Obtiene dado un TreePath con las capas devuelve un array con los objetos
     * Layer desde el inicio al final del path
     *
     * @param t TreePath
     *
     * @return Array de layers
     */
    private Layer[] getLayerByPath(String[] t) {
        ArrayList ret = new ArrayList();
        Layer raiz = root.getCapability().getLayer();
        ret.add(raiz);

        for (int i = 1; i < t.length; i++) {
            for (int j = 0; j < raiz.getLayerCount(); j++) {
            	if (raiz.getLayer(j).getTitle() == null) continue;
                if (raiz.getLayer(j).getTitle().getContent().equals(t[i])) {
                    raiz = raiz.getLayer(j);
                    ret.add(raiz);

                    break;
                }
            }
        }

        return (Layer[]) ret.toArray(new Layer[0]);
    }

    /**
     * @see com.iver.cit.gvsig.fmap.wms.wmsclient.WMSClient#doFeatureInfo(com.iver.cit.gvsig.fmap.wms.wmsclient.FeatureInfoQuery)
     */
    public byte[] doFeatureInfo(FeatureInfoQuery query)
        throws ValidationException, IOException, WMSException, 
            NoSuchFieldException {
        throw new RuntimeException("No debi llegar nunca a esta linea");
    }

    /**
     * @see com.iver.cit.gvsig.fmap.wms.WMSVersionClient#getMapURLs()
     */
    public String[] getMapURLs() {
        return mapURLs;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.wms.WMSVersionClient#parseException(byte[])
     */
    public String parseException(byte[] bytes) {
        String error = "";

        try {
            ServiceExceptionReport ex = (ServiceExceptionReport) ServiceExceptionReport.unmarshal(new InputStreamReader(
                        new ByteArrayInputStream(bytes)));
            ServiceException[] exs = ex.getServiceException();

            for (int i = 0; i < exs.length; i++) {
                if (exs[i].getCode() != null) {
                    error = error + exs[i].getCode() + ": ";
                }

                error += (exs[i].getContent() + "\n");
            }

            return error;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * @see com.iver.cit.gvsig.fmap.wms.WMSVersionClient#getInfoURLs()
     */
    public String[] getInfoURLs() {
        return infoURLs;
    }

	/**
	 * @see com.iver.wmsclient.WMSClient#getInfoFormats()
	 */
	public String[] getInfoFormats() {
		ArrayList formatos = new ArrayList();
		Format f = root.getCapability().getRequest().getFeatureInfo().getFormat();
		if (f.getGeoTIFF()!= null){
			formatos.add("GeoTIFF");
		}
		
		if (f.getBLANK()!= null){
			formatos.add("BLANK");
		}
		
		if (f.getGIF()!= null){
			formatos.add("GIF");
		}
		
		if (f.getGML_1()!= null){
			formatos.add("GML.1");
		}
		
		if (f.getGML_2()!= null){
			formatos.add("GML.2");
		}
		
		if (f.getGML_3()!= null){
			formatos.add("GeoGML.3");
		}
		
		if (f.getINIMAGE()!= null){
			formatos.add("INIMAGE");
		}
		
		if (f.getJPEG()!= null){
			formatos.add("JPEG");
		}
		
		if (f.getMIME()!= null){
			formatos.add("MIME");
		}
		
		if (f.getPNG()!= null){
			formatos.add("PNG");
		}
		
		if (f.getPPM()!= null){
			formatos.add("PPM");
		}
		
		if (f.getSVG()!= null){
			formatos.add("SVG");
		}

		if (f.getTIFF()!= null){
			formatos.add("TIFF");
		}

		if (f.getWBMP()!= null){
			formatos.add("WBMP");
		}

		if (f.getWebCGM()!= null){
			formatos.add("WebCGM");
		}

		if (f.getWMS_XML ()!= null){
			formatos.add("WMS_XML");
		}
		
		return (String[]) formatos.toArray(new String[0]);
	}

	/**
	 * @see com.iver.wmsclient.WMSVersionClient#getQueryHeader()
	 */
	public String getQueryHeader() {
		return "WMTVER=1.0.0&";
	}

	/**
	 * @see com.iver.wmsclient.WMSClient#createQuery()
	 */
	public MapQuery createQuery() {
        throw new RuntimeException("No debi llegar nunca a esta linea");
	}
}
